from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.postal_code_details_japan_post_postal_code_type import (
    PostalCodeDetailsJapanPostPostalCodeType,
)
from ..models.postal_code_details_japan_post_postal_entity import (
    PostalCodeDetailsJapanPostPostalEntity,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="PostalCodeDetailsJapanPost")


@_attrs_define
class PostalCodeDetailsJapanPost:
    """
    Attributes:
        postal_code (str): **ALPHA**

            7-digit business postal code in `XXX-XXXX` format (separated by dash)
        postal_entity (PostalCodeDetailsJapanPostPostalEntity | Unset): **ALPHA**

            The postal entity. This could be a governmental authority, a regulatory authority, or a designated postal
            operator.

            Description of supported values:

            - **ALPHA** `Japan Post`: Field indicating the data source (fixed as `Japan Post` for Japan)
        postal_code_type (PostalCodeDetailsJapanPostPostalCodeType | Unset): **ALPHA**

            The postal code type.

            Description of supported values:

            - **ALPHA** `ZIP`: Field describing the postal code type (fixed as `ZIP` for Japan)
        business_name (str | Unset): **ALPHA**

            Detailed information specifies company name, building name, floor number, etc. mapped to the corresponding
            business postal code
    """

    postal_code: str
    postal_entity: PostalCodeDetailsJapanPostPostalEntity | Unset = UNSET
    postal_code_type: PostalCodeDetailsJapanPostPostalCodeType | Unset = UNSET
    business_name: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        postal_code = self.postal_code

        postal_entity: str | Unset = UNSET
        if not isinstance(self.postal_entity, Unset):
            postal_entity = self.postal_entity.value

        postal_code_type: str | Unset = UNSET
        if not isinstance(self.postal_code_type, Unset):
            postal_code_type = self.postal_code_type.value

        business_name = self.business_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "postalCode": postal_code,
        })
        if postal_entity is not UNSET:
            field_dict["postalEntity"] = postal_entity
        if postal_code_type is not UNSET:
            field_dict["postalCodeType"] = postal_code_type
        if business_name is not UNSET:
            field_dict["businessName"] = business_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        postal_code = d.pop("postalCode")

        _postal_entity = d.pop("postalEntity", UNSET)
        postal_entity: PostalCodeDetailsJapanPostPostalEntity | Unset
        if isinstance(_postal_entity, Unset):
            postal_entity = UNSET
        else:
            postal_entity = PostalCodeDetailsJapanPostPostalEntity(_postal_entity)

        _postal_code_type = d.pop("postalCodeType", UNSET)
        postal_code_type: PostalCodeDetailsJapanPostPostalCodeType | Unset
        if isinstance(_postal_code_type, Unset):
            postal_code_type = UNSET
        else:
            postal_code_type = PostalCodeDetailsJapanPostPostalCodeType(
                _postal_code_type
            )

        business_name = d.pop("businessName", UNSET)

        postal_code_details_japan_post = cls(
            postal_code=postal_code,
            postal_entity=postal_entity,
            postal_code_type=postal_code_type,
            business_name=business_name,
        )

        postal_code_details_japan_post.additional_properties = d
        return postal_code_details_japan_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class V1RoutesCreateTransportMode(str, Enum):
    BICYCLE = "bicycle"
    BUS = "bus"
    CAR = "car"
    PEDESTRIAN = "pedestrian"
    PRIVATEBUS = "privateBus"
    SCOOTER = "scooter"
    TAXI = "taxi"
    TRUCK = "truck"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BrowseResultItemAdministrativeAreaType(str, Enum):
    COUNTRY = "country"
    COUNTY = "county"
    STATE = "state"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="FieldScore")


@_attrs_define
class FieldScore:
    """
    Attributes:
        country (float | Unset): Indicates how good the result country name or [ISO 3166-1 alpha-3] country code matches
            to the freeform or qualified input.
        country_code (float | Unset): Indicates how good the result [ISO 3166-1 alpha-3] country code matches to the
            freeform or qualified input.
        state (float | Unset): Indicates how good the result state name matches to the freeform or qualified input.
        state_code (float | Unset): Indicates how good the result state code matches to the freeform or qualified input.
        county (float | Unset): Indicates how good the result county name matches to the freeform or qualified input.
        county_code (float | Unset): Indicates how good the result county code matches to the freeform or qualified
            input.
        city (float | Unset): Indicates how good the result city name matches to the freeform or qualified input.
        district (float | Unset): Indicates how good the result district name matches to the freeform or qualified
            input.
        subdistrict (float | Unset): Indicates how good the result sub-district name matches to the freeform or
            qualified input.
        streets (list[float] | Unset): Indicates how good the result street names match to the freeform or qualified
            input. If the input
            contains multiple street names, the field score is calculated and returned for each of them
            individually.
        block (float | Unset): Indicates how good the result block name matches to the freeform or qualified input.
        subblock (float | Unset): Indicates how good the result sub-block name matches to the freeform or qualified
            input.
        house_number (float | Unset): Indicates how good the result house number matches to the freeform or qualified
            input. It may
            happen, that the house number, which one is looking for, is not yet in the map data. For such cases,
            the Geocode endpoint returns the nearest known house number on the same street. This represents the
            numeric difference between the requested and the returned house numbers.
        postal_code (float | Unset): Indicates how good the result postal code matches to the freeform or qualified
            input.
        building (float | Unset): Indicates how good the result building name matches to the freeform or qualified
            input.
        unit (float | Unset): Indicates whether a secondary unit (such as building, floor or suite) has been recognized
            in the freeform or qualified input.
        place_name (float | Unset): Indicates how good the result place name matches to the freeform or qualified input.
        ontology_name (float | Unset): Indicates how good the result ontology name matches to the freeform or qualified
            input.
    """

    country: float | Unset = UNSET
    country_code: float | Unset = UNSET
    state: float | Unset = UNSET
    state_code: float | Unset = UNSET
    county: float | Unset = UNSET
    county_code: float | Unset = UNSET
    city: float | Unset = UNSET
    district: float | Unset = UNSET
    subdistrict: float | Unset = UNSET
    streets: list[float] | Unset = UNSET
    block: float | Unset = UNSET
    subblock: float | Unset = UNSET
    house_number: float | Unset = UNSET
    postal_code: float | Unset = UNSET
    building: float | Unset = UNSET
    unit: float | Unset = UNSET
    place_name: float | Unset = UNSET
    ontology_name: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        country = self.country

        country_code = self.country_code

        state = self.state

        state_code = self.state_code

        county = self.county

        county_code = self.county_code

        city = self.city

        district = self.district

        subdistrict = self.subdistrict

        streets: list[float] | Unset = UNSET
        if not isinstance(self.streets, Unset):
            streets = self.streets

        block = self.block

        subblock = self.subblock

        house_number = self.house_number

        postal_code = self.postal_code

        building = self.building

        unit = self.unit

        place_name = self.place_name

        ontology_name = self.ontology_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if country is not UNSET:
            field_dict["country"] = country
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if state is not UNSET:
            field_dict["state"] = state
        if state_code is not UNSET:
            field_dict["stateCode"] = state_code
        if county is not UNSET:
            field_dict["county"] = county
        if county_code is not UNSET:
            field_dict["countyCode"] = county_code
        if city is not UNSET:
            field_dict["city"] = city
        if district is not UNSET:
            field_dict["district"] = district
        if subdistrict is not UNSET:
            field_dict["subdistrict"] = subdistrict
        if streets is not UNSET:
            field_dict["streets"] = streets
        if block is not UNSET:
            field_dict["block"] = block
        if subblock is not UNSET:
            field_dict["subblock"] = subblock
        if house_number is not UNSET:
            field_dict["houseNumber"] = house_number
        if postal_code is not UNSET:
            field_dict["postalCode"] = postal_code
        if building is not UNSET:
            field_dict["building"] = building
        if unit is not UNSET:
            field_dict["unit"] = unit
        if place_name is not UNSET:
            field_dict["placeName"] = place_name
        if ontology_name is not UNSET:
            field_dict["ontologyName"] = ontology_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        country = d.pop("country", UNSET)

        country_code = d.pop("countryCode", UNSET)

        state = d.pop("state", UNSET)

        state_code = d.pop("stateCode", UNSET)

        county = d.pop("county", UNSET)

        county_code = d.pop("countyCode", UNSET)

        city = d.pop("city", UNSET)

        district = d.pop("district", UNSET)

        subdistrict = d.pop("subdistrict", UNSET)

        streets = cast(list[float], d.pop("streets", UNSET))

        block = d.pop("block", UNSET)

        subblock = d.pop("subblock", UNSET)

        house_number = d.pop("houseNumber", UNSET)

        postal_code = d.pop("postalCode", UNSET)

        building = d.pop("building", UNSET)

        unit = d.pop("unit", UNSET)

        place_name = d.pop("placeName", UNSET)

        ontology_name = d.pop("ontologyName", UNSET)

        field_score = cls(
            country=country,
            country_code=country_code,
            state=state,
            state_code=state_code,
            county=county,
            county_code=county_code,
            city=city,
            district=district,
            subdistrict=subdistrict,
            streets=streets,
            block=block,
            subblock=subblock,
            house_number=house_number,
            postal_code=postal_code,
            building=building,
            unit=unit,
            place_name=place_name,
            ontology_name=ontology_name,
        )

        field_score.additional_properties = d
        return field_score

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import argparse
import datetime
import os
from typing import Final

TABLE_ORDER: Final = [
    "id",
    "subid",
    "inputStr",
    "label",
    "street",
    "houseNumber",
    "postalCode",
    "city",
    "county",
    "state",
    "countryName",
    "access_lat",
    "access_lng",
    "pos_lat",
    "pos_lng",
    "resultType",
    "houseNumberType",
    "countryCode",
    "stateCode",
    "countyCode",
    "scoreQuery",
    "score_country",
    "score_city",
    "score_houseNumber",
    "score_postalCode",
    "score_streets",
]

ALLOWED_COLUMNS: Final = {
    "id",
    "street",
    "housenumber",
    "postalcode",
    "city",
    "ctrycode",
    "address_string",
    "_",
}

GEOCODE_COST: Final = 3

SERVER: Final = "https://cg-license.casageo.eu/"

ADDRESS_NAMES_MODES: Final = ["matched", "normalized"]
POSTAL_CODE_MODES: Final = ["cityLookup", "districtLookup"]

RANGE_TYPES: Final = ["time", "distance"]
TRANSPORT_MODES: Final = ["car", "pedestrian", "bicycle"]
ROUTING_MODES: Final = ["fast", "short"]
DIRECTION_TYPES: Final = ["outgoing", "incoming"]

AVOIDABLE_FEATURES: Final = [
    "carShuttleTrain",
    "controlledAccessHighway",
    "dirtRoad",
    "ferry",
    "seasonalClosure",
    "tollRoad",
    "tunnel",
    # "uTurns", # Not supported for pedestrian, bicycle and scooter transport modes.
]

UNIT_SYSTEMS: Final = ["metric", "imperial"]


def get_client_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser("casageoclient")
    # TODO: Command Line Interface
    return parser


def get_coder_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser("casageocoder")
    parser.add_argument(
        "--output",
        help="File to output to if not given will default to stdout.",
        default=None,
    )
    parser.add_argument(
        "--indent",
        help="The indent of the output json if not given wont be indented",
        default=None,
        type=int,
    )
    parser.add_argument(
        "--file", type=str, default="", help="Provide a csv seperated by ;"
    )
    parser.add_argument(
        "--format",
        type=str,
        default="",
        help="Provide a file format, the format is like a csv header, but there are only a few valid column names.",
    )
    parser.add_argument("--street", type=str, default="", help="Provide the street")
    parser.add_argument(
        "--housenumber", type=str, default="", help="Provide the house number"
    )
    parser.add_argument(
        "--postalcode", type=str, default="", help="Provide the postal code"
    )
    parser.add_argument("--city", type=str, default="", help="Provide the city")
    parser.add_argument(
        "--ctrycode", type=str, default="", help="Provide the country code"
    )
    parser.add_argument(
        "--get-info",
        action="store_true",
        help="Prints info about your license key",
    )
    parser.add_argument(
        "--key",
        type=str,
        help="Provide the API key in an argument instead of a file",
        default=os.getenv("CASAGEO_KEY"),
    )
    parser.add_argument(
        "--addrstr", type=str, default="", help="Provide the entire query as one string"
    )
    parser.add_argument("--lang", type=str, default=None, help="Provide a language")
    parser.add_argument(
        "--json", action="store_true", help="output as a json instead of a csv"
    )

    return parser


def get_spatial_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser("casageospatial")
    parser.add_argument(
        "--output",
        help="File to output to if not given will default to stdout.",
        default=None,
    )
    parser.add_argument(
        "--indent",
        help="The indent of the output json if not given wont be indented",
        default=None,
        type=int,
    )
    parser.add_argument(
        "--file", type=str, default="", help="Provide a csv seperated by ;"
    )
    parser.add_argument(
        "--key",
        help="Provide a casageo spatial apikey.",
        default=os.getenv("CASAGEO_KEY"),
    )
    parser.add_argument(
        "--lang", "-l", default="en-US", help="Output language for the data."
    )

    subparsers = parser.add_subparsers(dest="command", required=True)
    routes_parser = subparsers.add_parser("routes")
    routes_parser.add_argument(
        "--raw",
        action="store_true",
        help="Output the raw json from the api instead of a geojson.",
    )

    routes_parser.add_argument(
        "--routing-mode",
        "-r",
        choices=ROUTING_MODES,
        default="fast",
    )
    routes_parser.add_argument(
        "--transport-mode",
        "-t",
        choices=TRANSPORT_MODES,
        default="car",
    )
    routes_parser.add_argument("pointa")
    routes_parser.add_argument("pointb")

    isolines_parser = subparsers.add_parser("isolines")

    isolines_parser.add_argument(
        "--routing-mode",
        "-r",
        choices=ROUTING_MODES,
        default="fast",
    )
    isolines_parser.add_argument(
        "--vehicle",
        "-v",
        choices=TRANSPORT_MODES,
        default="car",
    )
    isolines_parser.add_argument("--avoid", "-a", type=str)
    isolines_parser.add_argument(
        "--time",
        type=datetime.datetime.fromisoformat,
        help="Provide a date and time in ISO format.",
    )

    isolines_parser.add_argument(
        "--raw",
        action="store_true",
        help="Outputs the raw api response with minimal modifications.",
    )
    isolines_parser.add_argument("--rangetype", choices=RANGE_TYPES, default="time")
    isolines_parser.add_argument("point")
    isolines_parser.add_argument(
        "ranges",
        help="Provide comma seperated ranges in either meters or minutes depending on --range-type.",
    )

    # poi_parser = subparsers.add_parser("poi")
    #
    # poi_parser.add_argument("--limit", "-l", type=int, default=10)
    # poi_parser.add_argument(
    #     "--categories",
    #     "-c",
    #     type=str,
    #     default="100,200,300,400,500,600,700,800,900",
    #     help="https://www.here.com/docs/bundle/geocoding-and-search-api-developer-guide/page/topics-places/places-category-system-full.html",
    # )
    # poi_parser.add_argument("point", type=str)
    # poi_parser.add_argument("--raw", action="store_true", help="Output raw json")
    #
    # revgeocode_parser = subparsers.add_parser("revgeocode")
    # revgeocode_parser.add_argument("at", type=str, help="Specify a location.")
    # revgeocode_parser.add_argument("--raw", action="store_true", help="Output raw json")

    subparsers.add_parser("get-info")
    return parser



from enum import Enum


class GetRevgeocodeTypesItem(str, Enum):
    ADDRESS = "address"
    AREA = "area"
    CITY = "city"
    STREET = "street"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class Spans(str, Enum):
    BASEDURATION = "baseDuration"
    CARATTRIBUTES = "carAttributes"
    CONSUMPTION = "consumption"
    COUNTRYCODE = "countryCode"
    DURATION = "duration"
    DYNAMICSPEEDINFO = "dynamicSpeedInfo"
    FUNCTIONALCLASS = "functionalClass"
    GATES = "gates"
    INCIDENTS = "incidents"
    LENGTH = "length"
    MAXSPEED = "maxSpeed"
    NAMES = "names"
    NOTHROUGHRESTRICTIONS = "noThroughRestrictions"
    NOTICES = "notices"
    RAILWAYCROSSINGS = "railwayCrossings"
    ROUTENUMBERS = "routeNumbers"
    ROUTINGZONES = "routingZones"
    SCOOTERATTRIBUTES = "scooterAttributes"
    SEGMENTID = "segmentId"
    SEGMENTREF = "segmentRef"
    SPEEDLIMIT = "speedLimit"
    STATECODE = "stateCode"
    STREETATTRIBUTES = "streetAttributes"
    TOLLSYSTEMS = "tollSystems"
    TRUCKATTRIBUTES = "truckAttributes"
    TRUCKROADTYPES = "truckRoadTypes"
    TYPICALDURATION = "typicalDuration"
    WALKATTRIBUTES = "walkAttributes"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GetRevgeocodeShowItem(str, Enum):
    ADDRESSUSAGE = "addressUsage"
    COUNTRYINFO = "countryInfo"
    POSTALCODEDETAILS = "postalCodeDetails"
    STREETINFO = "streetInfo"
    TZ = "tz"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="IsolineErrorResponse")


@_attrs_define
class IsolineErrorResponse:
    """Response in case of error

    Attributes:
        title (str): Human-readable error description Example: Input data failed validation.
        status (int): HTTP status code Example: 400.
        code (str): Machine-readable service error code.

            All error codes of this service start with "`E605`". The last three digits describe a specific error. Provide
            this error code when contacting support.

            **NOTE:** The list of possible error codes could be extended in the future. The client application is expected
            to handle such a case gracefully.

            | Code      | Reason  |
            | --------- | ------- |
            | `E605010` | Invalid combination of truck and transport mode. Check `truck` for valid truck transport modes. |
            | `E605011` | Invalid combination of avoid feature `difficultTurns` or `uTurns` and transport mode. Check
            `avoid` for details. |
            | `E605012` | Invalid combination of transport mode and routing mode. Check `routingMode` for a list of
            supported combinations. |
            | `E605019` | `truck[weightPerAxle]` and `truck[weightPerAxleGroup]` are incompatible |
            | `E605020` | Invalid combination of `radius` and `snapRadius`  |
            | `E605030` | Invalid EV options. Check `ev` for details. |
            | `E605038` | Range based on consumption requires either ev or fuel parameters. Provide one as both parameters
            are missing. |
            | `E605040` | Invalid combination of EV and transport mode. Check `ev` for details. |
            | `E605041` | Invalid combination of EV and routing mode. Check `ev` for details. |
            | `E605048` | Invalid combination of avoid feature `difficultTurns` and truck category `lightTruck`. |
            | `E605070` | Invalid Range Type. |
            | `E605071` | Invalid Range for Range Type - Distance. |
            | `E605072` | Invalid Range for Range Type - Time. |
            | `E605073` | Invalid Shape Max Points. |
            | `E605074` | Invalid Range for Range Type - Consumption.  |
             Example: E605001.
        cause (str): Human-readable explanation for the error Example: The input data in question does not comply with
            validation rules.
        action (str): Human-readable description of the action that can be taken to correct the error Example: Request a
            valid id.
        correlation_id (str): Auto-generated id that univocally identifies the request Example:
            4199533b-6290-41db-8d79-edf4f4019a74.
    """

    title: str
    status: int
    code: str
    cause: str
    action: str
    correlation_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        status = self.status

        code = self.code

        cause = self.cause

        action = self.action

        correlation_id = self.correlation_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "title": title,
            "status": status,
            "code": code,
            "cause": cause,
            "action": action,
            "correlationId": correlation_id,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        title = d.pop("title")

        status = d.pop("status")

        code = d.pop("code")

        cause = d.pop("cause")

        action = d.pop("action")

        correlation_id = d.pop("correlationId")

        isoline_error_response = cls(
            title=title,
            status=status,
            code=code,
            cause=cause,
            action=action,
            correlation_id=correlation_id,
        )

        isoline_error_response.additional_properties = d
        return isoline_error_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EvChargingPoint")


@_attrs_define
class EvChargingPoint:
    """
    Attributes:
        number_of_connectors (int | Unset): Number of physical connectors in the connectors group
        volts_range (str | Unset): Voltage provided by the connectors group Example: 100-120V AC.
        phases (int | Unset): Number of phases provided by the connectors group
        amps_range (str | Unset): Amperage provided by the connectors group Example: 12A-80A.
        number_of_available (int | Unset): Number of available physical connectors in the connectors group (i.e not
            being used to charge a vehicle, out of service, etc)
        number_of_in_use (int | Unset): Number of occupied physical connectors in the connectors group.
        number_of_out_of_service (int | Unset): Number of physical connectors that are out of service at the charge
            point.
        number_of_reserved (int | Unset): Number of physical connectors that are reserved at the charge point.
        last_update_timestamp (str | Unset):
            Information about when the `numberOfAvailable` and `numberOfInUse` fields were last updated, in ISO 8601 format.
            If the time is UTC, a Z is added.
            If the time is not UTC , then the offset is added as a ±[hh][mm] value.
             Example: 2013-12-31T12:00:00.000Z.
    """

    number_of_connectors: int | Unset = UNSET
    volts_range: str | Unset = UNSET
    phases: int | Unset = UNSET
    amps_range: str | Unset = UNSET
    number_of_available: int | Unset = UNSET
    number_of_in_use: int | Unset = UNSET
    number_of_out_of_service: int | Unset = UNSET
    number_of_reserved: int | Unset = UNSET
    last_update_timestamp: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        number_of_connectors = self.number_of_connectors

        volts_range = self.volts_range

        phases = self.phases

        amps_range = self.amps_range

        number_of_available = self.number_of_available

        number_of_in_use = self.number_of_in_use

        number_of_out_of_service = self.number_of_out_of_service

        number_of_reserved = self.number_of_reserved

        last_update_timestamp = self.last_update_timestamp

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if number_of_connectors is not UNSET:
            field_dict["numberOfConnectors"] = number_of_connectors
        if volts_range is not UNSET:
            field_dict["voltsRange"] = volts_range
        if phases is not UNSET:
            field_dict["phases"] = phases
        if amps_range is not UNSET:
            field_dict["ampsRange"] = amps_range
        if number_of_available is not UNSET:
            field_dict["numberOfAvailable"] = number_of_available
        if number_of_in_use is not UNSET:
            field_dict["numberOfInUse"] = number_of_in_use
        if number_of_out_of_service is not UNSET:
            field_dict["numberOfOutOfService"] = number_of_out_of_service
        if number_of_reserved is not UNSET:
            field_dict["numberOfReserved"] = number_of_reserved
        if last_update_timestamp is not UNSET:
            field_dict["lastUpdateTimestamp"] = last_update_timestamp

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        number_of_connectors = d.pop("numberOfConnectors", UNSET)

        volts_range = d.pop("voltsRange", UNSET)

        phases = d.pop("phases", UNSET)

        amps_range = d.pop("ampsRange", UNSET)

        number_of_available = d.pop("numberOfAvailable", UNSET)

        number_of_in_use = d.pop("numberOfInUse", UNSET)

        number_of_out_of_service = d.pop("numberOfOutOfService", UNSET)

        number_of_reserved = d.pop("numberOfReserved", UNSET)

        last_update_timestamp = d.pop("lastUpdateTimestamp", UNSET)

        ev_charging_point = cls(
            number_of_connectors=number_of_connectors,
            volts_range=volts_range,
            phases=phases,
            amps_range=amps_range,
            number_of_available=number_of_available,
            number_of_in_use=number_of_in_use,
            number_of_out_of_service=number_of_out_of_service,
            number_of_reserved=number_of_reserved,
            last_update_timestamp=last_update_timestamp,
        )

        ev_charging_point.additional_properties = d
        return ev_charging_point

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LinkInfoSection")


@_attrs_define
class LinkInfoSection:
    """
    Attributes:
        cm_id (str): link ID in HERE core map
        side (str): The side of the link the attribute is applicable to

            - for `resultType=houseNumber` and `resultType=place`: side of the entity to the link
            - for `resultType=street`:
              - on endpoints *Geocode* and *Lookup*: side of the entity to the link, it is always "both"
              - on endpoints *Reverse Geocode* and *Multi-Reverse Geocode*: relative side from input to the link
    """

    cm_id: str
    side: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        cm_id = self.cm_id

        side = self.side

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "cmId": cm_id,
            "side": side,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        cm_id = d.pop("cmId")

        side = d.pop("side")

        link_info_section = cls(
            cm_id=cm_id,
            side=side,
        )

        link_info_section.additional_properties = d
        return link_info_section

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TimeRestrictedWeekdays(str, Enum):
    FR = "fr"
    MO = "mo"
    SA = "sa"
    SU = "su"
    TH = "th"
    TU = "tu"
    WE = "we"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tripadvisor_image_variants import TripadvisorImageVariants
    from ..models.tripadvisor_media_supplier import TripadvisorMediaSupplier


T = TypeVar("T", bound="TripadvisorImage")


@_attrs_define
class TripadvisorImage:
    """
    Attributes:
        href (str): The URL points to the default image provided by Tripadvisor (TM). These default images are typically
            sized for the *medium* format used by Tripadvisor.
            In rare cases, the image may be larger than expected. To ensure consistent rendering across all scenarios, we
            recommend enforcing maximum dimensions when displaying the image.
        supplier (TripadvisorMediaSupplier):
        variants (TripadvisorImageVariants | Unset):
    """

    href: str
    supplier: TripadvisorMediaSupplier
    variants: TripadvisorImageVariants | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        href = self.href

        supplier = self.supplier.to_dict()

        variants: dict[str, Any] | Unset = UNSET
        if not isinstance(self.variants, Unset):
            variants = self.variants.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "href": href,
            "supplier": supplier,
        })
        if variants is not UNSET:
            field_dict["variants"] = variants

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tripadvisor_image_variants import TripadvisorImageVariants
        from ..models.tripadvisor_media_supplier import TripadvisorMediaSupplier

        d = dict(src_dict)
        href = d.pop("href")

        supplier = TripadvisorMediaSupplier.from_dict(d.pop("supplier"))

        _variants = d.pop("variants", UNSET)
        variants: TripadvisorImageVariants | Unset
        if isinstance(_variants, Unset):
            variants = UNSET
        else:
            variants = TripadvisorImageVariants.from_dict(_variants)

        tripadvisor_image = cls(
            href=href,
            supplier=supplier,
            variants=variants,
        )

        tripadvisor_image.additional_properties = d
        return tripadvisor_image

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.range_price import RangePrice
    from ..models.single_price import SinglePrice


T = TypeVar("T", bound="TollCountrySummary")


@_attrs_define
class TollCountrySummary:
    """
    Attributes:
        country_code (str | Unset): ISO-3166-1 alpha-3 code Example: FRA.
        price (RangePrice | SinglePrice | Unset): Price of a fare
    """

    country_code: str | Unset = UNSET
    price: RangePrice | SinglePrice | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.single_price import SinglePrice

        country_code = self.country_code

        price: dict[str, Any] | Unset
        if isinstance(self.price, Unset):
            price = UNSET
        elif isinstance(self.price, SinglePrice):
            price = self.price.to_dict()
        else:
            price = self.price.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if price is not UNSET:
            field_dict["price"] = price

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.range_price import RangePrice
        from ..models.single_price import SinglePrice

        d = dict(src_dict)
        country_code = d.pop("countryCode", UNSET)

        def _parse_price(data: object) -> RangePrice | SinglePrice | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_fare_price_type_0 = SinglePrice.from_dict(data)

                return componentsschemas_fare_price_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_fare_price_type_1 = RangePrice.from_dict(data)

            return componentsschemas_fare_price_type_1

        price = _parse_price(d.pop("price", UNSET))

        toll_country_summary = cls(
            country_code=country_code,
            price=price,
        )

        toll_country_summary.additional_properties = d
        return toll_country_summary

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PostalCodeDetailsUspsZipPlus4RecordTypeCode(str, Enum):
    F = "F"
    G = "G"
    H = "H"
    P = "P"
    R = "R"
    S = "S"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="StreetInfo")


@_attrs_define
class StreetInfo:
    """
    Attributes:
        base_name (str | Unset): Base name part of the street name.
        street_type (str | Unset): Street type part of the street name.
        street_type_precedes (bool | Unset): Defines if the street type is before or after the base name.
        street_type_attached (bool | Unset): Defines if the street type is attached or unattached to the base name.
        prefix (str | Unset): A prefix is a directional identifier that precedes, but is not included in, the base name
            of a road.
        suffix (str | Unset): A suffix is a directional identifier that follows, but is not included in, the base name
            of a road.
        direction (str | Unset): Indicates the official directional identifiers assigned to highways, typically either
            "North/South" or "East/West"
        language (str | Unset): BCP 47 compliant language code
    """

    base_name: str | Unset = UNSET
    street_type: str | Unset = UNSET
    street_type_precedes: bool | Unset = UNSET
    street_type_attached: bool | Unset = UNSET
    prefix: str | Unset = UNSET
    suffix: str | Unset = UNSET
    direction: str | Unset = UNSET
    language: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        base_name = self.base_name

        street_type = self.street_type

        street_type_precedes = self.street_type_precedes

        street_type_attached = self.street_type_attached

        prefix = self.prefix

        suffix = self.suffix

        direction = self.direction

        language = self.language

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if base_name is not UNSET:
            field_dict["baseName"] = base_name
        if street_type is not UNSET:
            field_dict["streetType"] = street_type
        if street_type_precedes is not UNSET:
            field_dict["streetTypePrecedes"] = street_type_precedes
        if street_type_attached is not UNSET:
            field_dict["streetTypeAttached"] = street_type_attached
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if suffix is not UNSET:
            field_dict["suffix"] = suffix
        if direction is not UNSET:
            field_dict["direction"] = direction
        if language is not UNSET:
            field_dict["language"] = language

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        base_name = d.pop("baseName", UNSET)

        street_type = d.pop("streetType", UNSET)

        street_type_precedes = d.pop("streetTypePrecedes", UNSET)

        street_type_attached = d.pop("streetTypeAttached", UNSET)

        prefix = d.pop("prefix", UNSET)

        suffix = d.pop("suffix", UNSET)

        direction = d.pop("direction", UNSET)

        language = d.pop("language", UNSET)

        street_info = cls(
            base_name=base_name,
            street_type=street_type,
            street_type_precedes=street_type_precedes,
            street_type_attached=street_type_attached,
            prefix=prefix,
            suffix=suffix,
            direction=direction,
            language=language,
        )

        street_info.additional_properties = d
        return street_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.category_ref import CategoryRef


T = TypeVar("T", bound="AccessRestrictionAttributes")


@_attrs_define
class AccessRestrictionAttributes:
    """
    Attributes:
        categories (list[CategoryRef] | Unset): The list of place categories, this set of access restrictions refers to.
        restricted (bool | Unset): When set to `true`, the place response item has a known access restriction.
            When the place has no known access restriction, the `accessRestrictions` element is omitted, so `restricted`
            value is never set to `false`.
    """

    categories: list[CategoryRef] | Unset = UNSET
    restricted: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        categories: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()
                categories.append(categories_item)

        restricted = self.restricted

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if categories is not UNSET:
            field_dict["categories"] = categories
        if restricted is not UNSET:
            field_dict["restricted"] = restricted

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.category_ref import CategoryRef

        d = dict(src_dict)
        _categories = d.pop("categories", UNSET)
        categories: list[CategoryRef] | Unset = UNSET
        if _categories is not UNSET:
            categories = []
            for categories_item_data in _categories:
                categories_item = CategoryRef.from_dict(categories_item_data)

                categories.append(categories_item)

        restricted = d.pop("restricted", UNSET)

        access_restriction_attributes = cls(
            categories=categories,
            restricted=restricted,
        )

        access_restriction_attributes.additional_properties = d
        return access_restriction_attributes

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.attribution_link_type import AttributionLinkType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Attribution")


@_attrs_define
class Attribution:
    """The URL address to an external resource.

    Example:
        {'id': '88-7568-21.07.2023', 'text': 'Information for public transit provided by ThePublicTransit GmbH'}

    Attributes:
        id (str): Unique identifier for the web link. It is used to deduplicate links defined in multiple sections.
        text (str): Text describing the url address (e.g. The example website).
        href (str | Unset): An URL address that links to a particular resource. Example:
            https://url.address.com/resource.
        href_text (str | Unset): The interactive (or clickable) portion of the text. If not present (default), the
            entire content of the text attribute will be considered.
        type_ (AttributionLinkType | Unset): Attribution link type.

            * `disclaimer` - Page with further legal information from the agency.
            * `tariff` - Page with information on the tariff.
    """

    id: str
    text: str
    href: str | Unset = UNSET
    href_text: str | Unset = UNSET
    type_: AttributionLinkType | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        text = self.text

        href = self.href

        href_text = self.href_text

        type_: str | Unset = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "text": text,
        })
        if href is not UNSET:
            field_dict["href"] = href
        if href_text is not UNSET:
            field_dict["hrefText"] = href_text
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        text = d.pop("text")

        href = d.pop("href", UNSET)

        href_text = d.pop("hrefText", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: AttributionLinkType | Unset
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = AttributionLinkType(_type_)

        attribution = cls(
            id=id,
            text=text,
            href=href,
            href_text=href_text,
            type_=type_,
        )

        attribution.additional_properties = d
        return attribution

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.localized_route_number_direction import LocalizedRouteNumberDirection
from ..types import UNSET, Unset

T = TypeVar("T", bound="LocalizedRouteNumber")


@_attrs_define
class LocalizedRouteNumber:
    """Represents a route number in specific language with optional cardinal direction and route level.

    Example:
        {'value': 'US-101', 'language': 'en', 'direction': 'south', 'routeType': 1}

    Attributes:
        value (str): String written in the language specified in the language property.
        language (str | Unset): Language in BCP47 format
        direction (LocalizedRouteNumberDirection | Unset): This property indicates the official directional identifier
            assigned to highways. Use direction on sign in conjunction with official name or route number.
            For example, for route guidance, use "US-101 S" and not just "US-101" when appropriate.

            Note that the official direction is not necessarily the travel direction. For example, US-101 through the city
            of Sunnyvale is physically located East to West.
            However, the official direction on sign is North/South.
        route_type (int | Unset): Specifies route type for different route element. These values must be used in
            conjunction with a separate HERE data product: Country Profile Road Signs.
            One of the usage example for this property is it affects how the road shield will be rendered, i.e.
            this route type might change the shape, color and the font of the road shield.
            Related examples for different countries can be found [here](https://en.wikipedia.org/wiki/Highway_shield).
            Possible values are from 1 to 6. But if some countries add extra route type in the future, this range could
            change as well.
    """

    value: str
    language: str | Unset = UNSET
    direction: LocalizedRouteNumberDirection | Unset = UNSET
    route_type: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        value = self.value

        language = self.language

        direction: str | Unset = UNSET
        if not isinstance(self.direction, Unset):
            direction = self.direction.value

        route_type = self.route_type

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "value": value,
        })
        if language is not UNSET:
            field_dict["language"] = language
        if direction is not UNSET:
            field_dict["direction"] = direction
        if route_type is not UNSET:
            field_dict["routeType"] = route_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        value = d.pop("value")

        language = d.pop("language", UNSET)

        _direction = d.pop("direction", UNSET)
        direction: LocalizedRouteNumberDirection | Unset
        if isinstance(_direction, Unset):
            direction = UNSET
        else:
            direction = LocalizedRouteNumberDirection(_direction)

        route_type = d.pop("routeType", UNSET)

        localized_route_number = cls(
            value=value,
            language=language,
            direction=direction,
            route_type=route_type,
        )

        localized_route_number.additional_properties = d
        return localized_route_number

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="QueryTermResultItem")


@_attrs_define
class QueryTermResultItem:
    """
    Attributes:
        term (str): The term that will be suggested to the user.
        replaces (str): The sub-string of the original query that is replaced by this Query Term.
        start (int): The start index in codepoints (inclusive) of the text replaced in the original query.
        end (int): The end index in codepoints (exclusive) of the text replaced in the original query.
    """

    term: str
    replaces: str
    start: int
    end: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        term = self.term

        replaces = self.replaces

        start = self.start

        end = self.end

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "term": term,
            "replaces": replaces,
            "start": start,
            "end": end,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        term = d.pop("term")

        replaces = d.pop("replaces")

        start = d.pop("start")

        end = d.pop("end")

        query_term_result_item = cls(
            term=term,
            replaces=replaces,
            start=start,
            end=end,
        )

        query_term_result_item.additional_properties = d
        return query_term_result_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.localized_route_number import LocalizedRouteNumber


T = TypeVar("T", bound="SignpostLabelRouteNumber")


@_attrs_define
class SignpostLabelRouteNumber:
    """Route number on a signpost label. The `routeType` property of the LocalizedRouteNumber is always null at the moment
    for SignpostLabel, because the data is currently not available.

        Attributes:
            route_number (LocalizedRouteNumber | Unset): Represents a route number in specific language with optional
                cardinal direction and route level. Example: {'value': 'US-101', 'language': 'en', 'direction': 'south',
                'routeType': 1}.
    """

    route_number: LocalizedRouteNumber | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        route_number: dict[str, Any] | Unset = UNSET
        if not isinstance(self.route_number, Unset):
            route_number = self.route_number.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if route_number is not UNSET:
            field_dict["routeNumber"] = route_number

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.localized_route_number import LocalizedRouteNumber

        d = dict(src_dict)
        _route_number = d.pop("routeNumber", UNSET)
        route_number: LocalizedRouteNumber | Unset
        if isinstance(_route_number, Unset):
            route_number = UNSET
        else:
            route_number = LocalizedRouteNumber.from_dict(_route_number)

        signpost_label_route_number = cls(
            route_number=route_number,
        )

        signpost_label_route_number.additional_properties = d
        return signpost_label_route_number

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tripadvisor_media_supplier import TripadvisorMediaSupplier


T = TypeVar("T", bound="Editorial")


@_attrs_define
class Editorial:
    """
    Attributes:
        description (str): The editorial content.
        language (str): A language code indicating what language the editorial is in, if known.
        supplier (TripadvisorMediaSupplier):
        href (str | Unset): An optional deep link to a 3rd party source providing the editorials.
    """

    description: str
    language: str
    supplier: TripadvisorMediaSupplier
    href: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        description = self.description

        language = self.language

        supplier = self.supplier.to_dict()

        href = self.href

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "description": description,
            "language": language,
            "supplier": supplier,
        })
        if href is not UNSET:
            field_dict["href"] = href

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tripadvisor_media_supplier import TripadvisorMediaSupplier

        d = dict(src_dict)
        description = d.pop("description")

        language = d.pop("language")

        supplier = TripadvisorMediaSupplier.from_dict(d.pop("supplier"))

        href = d.pop("href", UNSET)

        editorial = cls(
            description=description,
            language=language,
            supplier=supplier,
            href=href,
        )

        editorial.additional_properties = d
        return editorial

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MicroPointAddressSection")


@_attrs_define
class MicroPointAddressSection:
    """
    Attributes:
        hmc_id (str): The micro point address ID in HERE Map Content
        partition_id (str | Unset): **ALPHA**

            Unique identifier that references a specific subset of geo-location related layers
            (e.g., address, location, street, or place) in the HERE Map Content
    """

    hmc_id: str
    partition_id: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        hmc_id = self.hmc_id

        partition_id = self.partition_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "hmcId": hmc_id,
        })
        if partition_id is not UNSET:
            field_dict["partitionId"] = partition_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        hmc_id = d.pop("hmcId")

        partition_id = d.pop("partitionId", UNSET)

        micro_point_address_section = cls(
            hmc_id=hmc_id,
            partition_id=partition_id,
        )

        micro_point_address_section.additional_properties = d
        return micro_point_address_section

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.admin_id_section import AdminIdSection
    from ..models.cm_version_section import CmVersionSection
    from ..models.link_info_section import LinkInfoSection
    from ..models.micro_point_address_section import MicroPointAddressSection
    from ..models.point_address_section import PointAddressSection
    from ..models.segment import Segment


T = TypeVar("T", bound="MapReferenceSection")


@_attrs_define
class MapReferenceSection:
    """
    Attributes:
        links (list[LinkInfoSection] | Unset): An identifier for link and the side of link the attribute is applicable
            to
        point_address (PointAddressSection | Unset):
        micro_point_address (MicroPointAddressSection | Unset):
        segments (list[Segment] | Unset): The section containing the segment references
        country (AdminIdSection | Unset):
        state (AdminIdSection | Unset):
        county (AdminIdSection | Unset):
        city (AdminIdSection | Unset):
        district (AdminIdSection | Unset):
        subdistrict (AdminIdSection | Unset):
        cm_version (CmVersionSection | Unset):
    """

    links: list[LinkInfoSection] | Unset = UNSET
    point_address: PointAddressSection | Unset = UNSET
    micro_point_address: MicroPointAddressSection | Unset = UNSET
    segments: list[Segment] | Unset = UNSET
    country: AdminIdSection | Unset = UNSET
    state: AdminIdSection | Unset = UNSET
    county: AdminIdSection | Unset = UNSET
    city: AdminIdSection | Unset = UNSET
    district: AdminIdSection | Unset = UNSET
    subdistrict: AdminIdSection | Unset = UNSET
    cm_version: CmVersionSection | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        links: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.links, Unset):
            links = []
            for links_item_data in self.links:
                links_item = links_item_data.to_dict()
                links.append(links_item)

        point_address: dict[str, Any] | Unset = UNSET
        if not isinstance(self.point_address, Unset):
            point_address = self.point_address.to_dict()

        micro_point_address: dict[str, Any] | Unset = UNSET
        if not isinstance(self.micro_point_address, Unset):
            micro_point_address = self.micro_point_address.to_dict()

        segments: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.segments, Unset):
            segments = []
            for segments_item_data in self.segments:
                segments_item = segments_item_data.to_dict()
                segments.append(segments_item)

        country: dict[str, Any] | Unset = UNSET
        if not isinstance(self.country, Unset):
            country = self.country.to_dict()

        state: dict[str, Any] | Unset = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.to_dict()

        county: dict[str, Any] | Unset = UNSET
        if not isinstance(self.county, Unset):
            county = self.county.to_dict()

        city: dict[str, Any] | Unset = UNSET
        if not isinstance(self.city, Unset):
            city = self.city.to_dict()

        district: dict[str, Any] | Unset = UNSET
        if not isinstance(self.district, Unset):
            district = self.district.to_dict()

        subdistrict: dict[str, Any] | Unset = UNSET
        if not isinstance(self.subdistrict, Unset):
            subdistrict = self.subdistrict.to_dict()

        cm_version: dict[str, Any] | Unset = UNSET
        if not isinstance(self.cm_version, Unset):
            cm_version = self.cm_version.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if links is not UNSET:
            field_dict["links"] = links
        if point_address is not UNSET:
            field_dict["pointAddress"] = point_address
        if micro_point_address is not UNSET:
            field_dict["microPointAddress"] = micro_point_address
        if segments is not UNSET:
            field_dict["segments"] = segments
        if country is not UNSET:
            field_dict["country"] = country
        if state is not UNSET:
            field_dict["state"] = state
        if county is not UNSET:
            field_dict["county"] = county
        if city is not UNSET:
            field_dict["city"] = city
        if district is not UNSET:
            field_dict["district"] = district
        if subdistrict is not UNSET:
            field_dict["subdistrict"] = subdistrict
        if cm_version is not UNSET:
            field_dict["cmVersion"] = cm_version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.admin_id_section import AdminIdSection
        from ..models.cm_version_section import CmVersionSection
        from ..models.link_info_section import LinkInfoSection
        from ..models.micro_point_address_section import MicroPointAddressSection
        from ..models.point_address_section import PointAddressSection
        from ..models.segment import Segment

        d = dict(src_dict)
        _links = d.pop("links", UNSET)
        links: list[LinkInfoSection] | Unset = UNSET
        if _links is not UNSET:
            links = []
            for links_item_data in _links:
                links_item = LinkInfoSection.from_dict(links_item_data)

                links.append(links_item)

        _point_address = d.pop("pointAddress", UNSET)
        point_address: PointAddressSection | Unset
        if isinstance(_point_address, Unset):
            point_address = UNSET
        else:
            point_address = PointAddressSection.from_dict(_point_address)

        _micro_point_address = d.pop("microPointAddress", UNSET)
        micro_point_address: MicroPointAddressSection | Unset
        if isinstance(_micro_point_address, Unset):
            micro_point_address = UNSET
        else:
            micro_point_address = MicroPointAddressSection.from_dict(
                _micro_point_address
            )

        _segments = d.pop("segments", UNSET)
        segments: list[Segment] | Unset = UNSET
        if _segments is not UNSET:
            segments = []
            for segments_item_data in _segments:
                segments_item = Segment.from_dict(segments_item_data)

                segments.append(segments_item)

        _country = d.pop("country", UNSET)
        country: AdminIdSection | Unset
        if isinstance(_country, Unset):
            country = UNSET
        else:
            country = AdminIdSection.from_dict(_country)

        _state = d.pop("state", UNSET)
        state: AdminIdSection | Unset
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = AdminIdSection.from_dict(_state)

        _county = d.pop("county", UNSET)
        county: AdminIdSection | Unset
        if isinstance(_county, Unset):
            county = UNSET
        else:
            county = AdminIdSection.from_dict(_county)

        _city = d.pop("city", UNSET)
        city: AdminIdSection | Unset
        if isinstance(_city, Unset):
            city = UNSET
        else:
            city = AdminIdSection.from_dict(_city)

        _district = d.pop("district", UNSET)
        district: AdminIdSection | Unset
        if isinstance(_district, Unset):
            district = UNSET
        else:
            district = AdminIdSection.from_dict(_district)

        _subdistrict = d.pop("subdistrict", UNSET)
        subdistrict: AdminIdSection | Unset
        if isinstance(_subdistrict, Unset):
            subdistrict = UNSET
        else:
            subdistrict = AdminIdSection.from_dict(_subdistrict)

        _cm_version = d.pop("cmVersion", UNSET)
        cm_version: CmVersionSection | Unset
        if isinstance(_cm_version, Unset):
            cm_version = UNSET
        else:
            cm_version = CmVersionSection.from_dict(_cm_version)

        map_reference_section = cls(
            links=links,
            point_address=point_address,
            micro_point_address=micro_point_address,
            segments=segments,
            country=country,
            state=state,
            county=county,
            city=city,
            district=district,
            subdistrict=subdistrict,
            cm_version=cm_version,
        )

        map_reference_section.additional_properties = d
        return map_reference_section

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ev_availability_attributes import EvAvailabilityAttributes
    from ..models.ev_charging_attributes import EvChargingAttributes
    from ..models.fuel_station_attributes import FuelStationAttributes
    from ..models.truck_amenity_generic import TruckAmenityGeneric
    from ..models.truck_amenity_showers import TruckAmenityShowers


T = TypeVar("T", bound="ExtendedAttribute")


@_attrs_define
class ExtendedAttribute:
    """
    Attributes:
        ev_station (EvChargingAttributes | Unset):
        ev_availability (EvAvailabilityAttributes | Unset):
        fuel_station (FuelStationAttributes | Unset):
        truck_amenities (list[TruckAmenityGeneric | TruckAmenityShowers] | Unset): **BETA, RESTRICTED**

            This field contains a listing of acceptable key-value pairs for amenities at places useful for truck drivers.
    """

    ev_station: EvChargingAttributes | Unset = UNSET
    ev_availability: EvAvailabilityAttributes | Unset = UNSET
    fuel_station: FuelStationAttributes | Unset = UNSET
    truck_amenities: list[TruckAmenityGeneric | TruckAmenityShowers] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.truck_amenity_generic import TruckAmenityGeneric

        ev_station: dict[str, Any] | Unset = UNSET
        if not isinstance(self.ev_station, Unset):
            ev_station = self.ev_station.to_dict()

        ev_availability: dict[str, Any] | Unset = UNSET
        if not isinstance(self.ev_availability, Unset):
            ev_availability = self.ev_availability.to_dict()

        fuel_station: dict[str, Any] | Unset = UNSET
        if not isinstance(self.fuel_station, Unset):
            fuel_station = self.fuel_station.to_dict()

        truck_amenities: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.truck_amenities, Unset):
            truck_amenities = []
            for truck_amenities_item_data in self.truck_amenities:
                truck_amenities_item: dict[str, Any]
                if isinstance(truck_amenities_item_data, TruckAmenityGeneric):
                    truck_amenities_item = truck_amenities_item_data.to_dict()
                else:
                    truck_amenities_item = truck_amenities_item_data.to_dict()

                truck_amenities.append(truck_amenities_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if ev_station is not UNSET:
            field_dict["evStation"] = ev_station
        if ev_availability is not UNSET:
            field_dict["evAvailability"] = ev_availability
        if fuel_station is not UNSET:
            field_dict["fuelStation"] = fuel_station
        if truck_amenities is not UNSET:
            field_dict["truckAmenities"] = truck_amenities

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ev_availability_attributes import EvAvailabilityAttributes
        from ..models.ev_charging_attributes import EvChargingAttributes
        from ..models.fuel_station_attributes import FuelStationAttributes
        from ..models.truck_amenity_generic import TruckAmenityGeneric
        from ..models.truck_amenity_showers import TruckAmenityShowers

        d = dict(src_dict)
        _ev_station = d.pop("evStation", UNSET)
        ev_station: EvChargingAttributes | Unset
        if isinstance(_ev_station, Unset):
            ev_station = UNSET
        else:
            ev_station = EvChargingAttributes.from_dict(_ev_station)

        _ev_availability = d.pop("evAvailability", UNSET)
        ev_availability: EvAvailabilityAttributes | Unset
        if isinstance(_ev_availability, Unset):
            ev_availability = UNSET
        else:
            ev_availability = EvAvailabilityAttributes.from_dict(_ev_availability)

        _fuel_station = d.pop("fuelStation", UNSET)
        fuel_station: FuelStationAttributes | Unset
        if isinstance(_fuel_station, Unset):
            fuel_station = UNSET
        else:
            fuel_station = FuelStationAttributes.from_dict(_fuel_station)

        _truck_amenities = d.pop("truckAmenities", UNSET)
        truck_amenities: list[TruckAmenityGeneric | TruckAmenityShowers] | Unset = UNSET
        if _truck_amenities is not UNSET:
            truck_amenities = []
            for truck_amenities_item_data in _truck_amenities:

                def _parse_truck_amenities_item(
                    data: object,
                ) -> TruckAmenityGeneric | TruckAmenityShowers:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        truck_amenities_item_type_0 = TruckAmenityGeneric.from_dict(
                            data
                        )

                        return truck_amenities_item_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    truck_amenities_item_type_1 = TruckAmenityShowers.from_dict(data)

                    return truck_amenities_item_type_1

                truck_amenities_item = _parse_truck_amenities_item(
                    truck_amenities_item_data
                )

                truck_amenities.append(truck_amenities_item)

        extended_attribute = cls(
            ev_station=ev_station,
            ev_availability=ev_availability,
            fuel_station=fuel_station,
            truck_amenities=truck_amenities,
        )

        extended_attribute.additional_properties = d
        return extended_attribute

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.truck_amenity_generic_type import TruckAmenityGenericType

T = TypeVar("T", bound="TruckAmenityGeneric")


@_attrs_define
class TruckAmenityGeneric:
    """
    Attributes:
        type_ (TruckAmenityGenericType): **BETA, RESTRICTED**

            The kind of amenity

            Description of supported values:

            - **BETA, RESTRICTED** `carWash`: car washing and/or detailing services
            - **BETA, RESTRICTED** `chemicalToiletDisposal`: a waste disposal station, for use in emptying and cleaning
            chemical toilets
            - **BETA, RESTRICTED** `convenienceStore`: a convenience store can be found at the location
            - **BETA, RESTRICTED** `highCanopy`: High canopy at fueling station/truck stop. This references the canopy
            height over the pumps being at a height for most trucks to enter.
            - **BETA, RESTRICTED** `idleReduction`: idle reduction system onsite
            - **BETA, RESTRICTED** `nightParkingOnly`: parking is restricted to night hours only
            - **BETA, RESTRICTED** `paidParking`: A fee is required to use the truck parking space.
            - **BETA, RESTRICTED** `parking`: truck parking onsite or nearby
            - **BETA, RESTRICTED** `powerSupply`: power supply, for use in maintaining the temperature of refrigerated
            trucks
            - **BETA, RESTRICTED** `reservableParking`: The truck parking space requires advance booking for a defined time
            interval.
            - **BETA, RESTRICTED** `secureParking`: secure parking location
            - **BETA, RESTRICTED** `truckScales`: truck scales onsite or nearby
            - **BETA, RESTRICTED** `truckService`: truck service onsite or nearby
            - **BETA, RESTRICTED** `truckStop`: a truck stop
            - **BETA, RESTRICTED** `truckWash`: truck wash onsite or nearby
            - **BETA, RESTRICTED** `wifi`: Wi-Fi access
        available (bool): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that this amenity is available at this place
            - `false` indicates that this amenity is not available at this place
    """

    type_: TruckAmenityGenericType
    available: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        available = self.available

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "available": available,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = TruckAmenityGenericType(d.pop("type"))

        available = d.pop("available")

        truck_amenity_generic = cls(
            type_=type_,
            available=available,
        )

        truck_amenity_generic.additional_properties = d
        return truck_amenity_generic

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.name_type import NameType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Name")


@_attrs_define
class Name:
    """
    Attributes:
        value (str): The text value of this name.
        type_ (NameType): The type of this name.

            Description of supported values:

            - `abbreviation`: Abbreviation is a shortened form of the official name.  For example, "CA" for "California".
            Abbreviation may also be an area code.
            - `areaCode`: Area code is a code representation of an admin. For example for countries this can be an ISO-3 or
            ISO-2 Code: "USA", "US" for "United States".
            - `baseName`: Base name represents the official name in one of the country official languages. For example,
            "Frankfurt am Main".
            - `exonym`: Exonym is a translation of the official name to a language that is not one of the country official
            languages. For example "Francfort-sur-le-Main" for "Frankfurt am Main".
            - `shortened`: A shortened variant of a long name where optional parts are omitted. For example, "Bad Soden" for
            "Bad Soden am Taunus".
            - `synonym`: A synonym is an additional commonly used name that is different than the official name, in the same
            language as the official name. For example, "Frankfurt" for "Frankfurt am Main".
        language (str | Unset): The [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) language code.
        primary (bool | Unset): Flag indicating whether it is a primary name. This is the name which is exposed in the
            result by default. This field is visible only when the value is `true`.
        transliterated (bool | Unset): Flag indicating whether the name is transliterated. Additional transliterated
            form of the name is included for all non-Latin-1 names. This field is visible only when the value is `true`.
    """

    value: str
    type_: NameType
    language: str | Unset = UNSET
    primary: bool | Unset = UNSET
    transliterated: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        value = self.value

        type_ = self.type_.value

        language = self.language

        primary = self.primary

        transliterated = self.transliterated

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "value": value,
            "type": type_,
        })
        if language is not UNSET:
            field_dict["language"] = language
        if primary is not UNSET:
            field_dict["primary"] = primary
        if transliterated is not UNSET:
            field_dict["transliterated"] = transliterated

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        value = d.pop("value")

        type_ = NameType(d.pop("type"))

        language = d.pop("language", UNSET)

        primary = d.pop("primary", UNSET)

        transliterated = d.pop("transliterated", UNSET)

        name = cls(
            value=value,
            type_=type_,
            language=language,
            primary=primary,
            transliterated=transliterated,
        )

        name.additional_properties = d
        return name

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="RoutingErrorResponse")


@_attrs_define
class RoutingErrorResponse:
    """Response in case of error

    Attributes:
        title (str): Human-readable error description Example: Input data failed validation.
        status (int): HTTP status code Example: 400.
        code (str): Machine readable service error code.

            All error codes of this service start with "`E605`". The last three digits describe a specific error. Provide
            this error code when contacting support.

            **NOTE:** Please note that the list of possible error codes could be extended in the future. The client
            application is expected to handle such a case gracefully.

            | Code      | Reason  |
            | --------- | ------- |
            | `E60500X` | Malformed query. Typically due to invalid values such as `transportMode=spaceShuttle` or missing
            required fields. Check the error message for details. |
            | `E605010` | Invalid combination of vehicle options and transport mode. |
            | `E605011` | Invalid combination of avoid feature `difficultTurns` or `uTurns` and transport mode. Check
            `avoid` for details. |
            | `E605012` | Invalid combination of transport mode and routing mode. Check `routingMode` for a list of
            supported combinations. |
            | `E605013` | Invalid return options. Check `return` for valid combinations of values. |
            | `E605014` | Invalid language code. Check `lang` for details on how valid language codes look. |
            | `E605015` | Too many alternatives. Check `alternatives` for the maximum number of alternatives allowed. |
            | `E605016` | Invalid country code. Check `countries` under `Exclude`. |
            | `E605017` | `spans` contains a value whose dependency has not been requested |
            | `E605018` | Invalid combination of departure and arrival time |
            | `E605019` | `truck[weightPerAxle]` and `truck[weightPerAxleGroup]` are incompatible |
            | `E605020` | Invalid combination of `radius` and `snapRadius`  |
            | `E605021` | Invalid Combination of `vehicle[occupancy]`/`allow[hot]`/`allow[hov]` and `vehicle[hovOccupancy]`
            |
            | `E605022` | `radius` * `radiusPenalty` / 100 must not exceed 7200  |
            | `E605023` | Invalid combination of `radiusPenalty` and `snapRadius` |
            | `E605024` | `vehicle[currentWeight]` or `vehicle[grossWeight]` exceeds limit for given `transportMode`. |
            | `E605025` | Invalid state code. Check `states` under `Exclude` |
            | `E605026` | Invalid country code. Check `states` under `Exclude` |
            | `E605027` | Ev parameters cannot be used with the specified transportMode. |
            | `E605028` | Fuel parameters cannot be used with the specified transportMode. |
            | `E605029` | State code not available in map data. Check `states` under `Exclude` |
            | `E605030` | Invalid EV options. Check `ev` for details. |
            | `E605032` | Invalid transport mode for speed cap, check `vehicle[speedCap]` for details. |
            | `E605033` | Invalid combination of scooter and transport mode. Check `scooter` for valid scooter transport
            modes. |
            | `E605034` | Invalid Speed Cap, check `vehicle[speedCap]` or `vehicle[speedCapPerFc]` for details. |
            | `E605035` | MLDuration is not supported with `vehicle[speedCap]` and `vehicle[speedCapPerFc]` parameter. |
            | `E605036` | Consumption parameters are not supported for combination of EV and Fuel-based vehicle. Check `ev`
            or `fuel` for details. |
            | `E605037` | Invalid Fuel options. Check `fuel` for details. |
            | `E605039` | Invalid combination of driver schedule and transport mode. |
            | `E605040` | Invalid combination of `ev[makeReachable]` and `transportMode`. Check `ev` for details. |
            | `E605041` | Invalid combination of `ev[makeReachable]` and `routingMode`. Check `ev` for details. |
            | `E605043` | Invalid combination of `ev[makeReachable]` and `avoid` options. Check `ev` for details. |
            | `E605047` | Invalid combination of `ev[makeReachable]` and `arrivalTime`. Check `ev` for details. |
            | `E605048` | Invalid combination of avoid feature `difficultTurns` and truck category `lightTruck`. |
            | `E605052` | Invalid number of trace points. Check `MatchTrace` for the minimum and maximum number of trace
            points allowed. |
            | `E605053` | Invalid Match trace via. Check `via` in `MatchTrace` for valid indexes. |
            | `E605054` | Too many avoid areas. Check `areas` for the maximum number of avoid areas allowed. |
            | `E605055` | Invalid trailer axle count. |
            | `E605056` | Too many avoid polygons. Check `areas[polygon]` for the maximum number of polygons allowed. |
            | `E605057` | Too many vertices in the polygon. Check `areas[polygon]` for the maximum number of vertices
            allowed. |
            | `E605058` | Not enough vertices in the polygon. Check `areas[polygon]` for the minimum number of vertices
            allowed. |
            | `E605059` | Polygon is self-intersecting. Check `areas[polygon]`. |
            | `E605060` | Invalid trace point. Check `lat` value must be between -90 and 90 and `lng` value must be between
            -180 and 180. |
            | `E605064` | Invalid transport mode for speed cap per Functional Class, check `vehicle[speedCapPerFc]` for
            details. |
            | `E605075` | Invalid customizationIndex. |
            | `E605076` | Too many charging stations to exclude. Check `ev[excludeChargingStations]` for maximum number of
            charging stations allowed. |
            | `E605077` | `rerouting[lastTraveledSectionIndex]` is out of range. |
            | `E605101` | Credentials not allowed for calculating routes in Japan. |
            | `E605201` | RouteHandle not valid (anymore). (Re-)calculate route to retrieve new handle. |
            | `E605204` | Invalid combination of EV and driver schedule. |
            | `E605301` | Pedestrian options are only supported for transport mode `pedestrian`. |
            | `E605302` | Routing zones is not supported for transport mode `pedestrian`. |
            | `E605303` | Avoiding routing zones is not supported for transport mode `pedestrian`. |
            | `E605304` | Avoiding truck road types is not supported for transport mode `pedestrian`. |
            | `E605400` | Customization not supported. |
            | `E6055XX` | Internal server error. |
             Example: E605001.
        cause (str): Human-readable explanation for the error Example: The input data in question does not comply with
            validation rules.
        action (str): Human-readable description of the action that can be taken to correct the error Example: Request a
            valid id.
        correlation_id (str): Auto-generated id that univocally identifies the request Example:
            4199533b-6290-41db-8d79-edf4f4019a74.
    """

    title: str
    status: int
    code: str
    cause: str
    action: str
    correlation_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        status = self.status

        code = self.code

        cause = self.cause

        action = self.action

        correlation_id = self.correlation_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "title": title,
            "status": status,
            "code": code,
            "cause": cause,
            "action": action,
            "correlationId": correlation_id,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        title = d.pop("title")

        status = d.pop("status")

        code = d.pop("code")

        cause = d.pop("cause")

        action = d.pop("action")

        correlation_id = d.pop("correlationId")

        routing_error_response = cls(
            title=title,
            status=status,
            code=code,
            cause=cause,
            action=action,
            correlation_id=correlation_id,
        )

        routing_error_response.additional_properties = d
        return routing_error_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="CmVersionSection")


@_attrs_define
class CmVersionSection:
    """
    Attributes:
        region (str): Region ID
        dvn (str): Map version details (DVN) containing the base line for the map schema and an identifier for the
            weekly or quarterly update.

            - Format: YYQ<weekly/quaterly update>
              - Example: 23150 (map schema: Q1/2023, weekly update 50)
    """

    region: str
    dvn: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        region = self.region

        dvn = self.dvn

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "region": region,
            "dvn": dvn,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        region = d.pop("region")

        dvn = d.pop("dvn")

        cm_version_section = cls(
            region=region,
            dvn=dvn,
        )

        cm_version_section.additional_properties = d
        return cm_version_section

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.v1_routes_create_routing_mode import V1RoutesCreateRoutingMode
from ...models.v1_routes_create_transport_mode import V1RoutesCreateTransportMode
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    arrival_time: Union[Unset, str] = UNSET,
    departure_time: Union[Unset, str] = UNSET,
    destination: str,
    origin: str,
    return_: Union[Unset, str] = UNSET,
    routing_mode: Union[
        Unset, V1RoutesCreateRoutingMode
    ] = V1RoutesCreateRoutingMode.FAST,
    transport_mode: V1RoutesCreateTransportMode,
    via: Union[Unset, str] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["arrivalTime"] = arrival_time

    params["departureTime"] = departure_time

    params["destination"] = destination

    params["origin"] = origin

    params["return"] = return_

    json_routing_mode: Union[Unset, str] = UNSET
    if not isinstance(routing_mode, Unset):
        json_routing_mode = routing_mode.value

    params["routingMode"] = json_routing_mode

    json_transport_mode = transport_mode.value
    params["transportMode"] = json_transport_mode

    params["via"] = via

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/api/v1/routes",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Any]:
    if response.status_code == 200:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    arrival_time: Union[Unset, str] = UNSET,
    departure_time: Union[Unset, str] = UNSET,
    destination: str,
    origin: str,
    return_: Union[Unset, str] = UNSET,
    routing_mode: Union[
        Unset, V1RoutesCreateRoutingMode
    ] = V1RoutesCreateRoutingMode.FAST,
    transport_mode: V1RoutesCreateTransportMode,
    via: Union[Unset, str] = UNSET,
) -> Response[Any]:
    """Routes

    Args:
        arrival_time (Union[Unset, str]):
        departure_time (Union[Unset, str]):
        destination (str):
        origin (str):
        return_ (Union[Unset, str]):
        routing_mode (Union[Unset, V1RoutesCreateRoutingMode]):  Default:
            V1RoutesCreateRoutingMode.FAST.
        transport_mode (V1RoutesCreateTransportMode):
        via (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        arrival_time=arrival_time,
        departure_time=departure_time,
        destination=destination,
        origin=origin,
        return_=return_,
        routing_mode=routing_mode,
        transport_mode=transport_mode,
        via=via,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    arrival_time: Union[Unset, str] = UNSET,
    departure_time: Union[Unset, str] = UNSET,
    destination: str,
    origin: str,
    return_: Union[Unset, str] = UNSET,
    routing_mode: Union[
        Unset, V1RoutesCreateRoutingMode
    ] = V1RoutesCreateRoutingMode.FAST,
    transport_mode: V1RoutesCreateTransportMode,
    via: Union[Unset, str] = UNSET,
) -> Response[Any]:
    """Routes

    Args:
        arrival_time (Union[Unset, str]):
        departure_time (Union[Unset, str]):
        destination (str):
        origin (str):
        return_ (Union[Unset, str]):
        routing_mode (Union[Unset, V1RoutesCreateRoutingMode]):  Default:
            V1RoutesCreateRoutingMode.FAST.
        transport_mode (V1RoutesCreateTransportMode):
        via (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        arrival_time=arrival_time,
        departure_time=departure_time,
        destination=destination,
        origin=origin,
        return_=return_,
        routing_mode=routing_mode,
        transport_mode=transport_mode,
        via=via,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EVPhysicalModel")


@_attrs_define
class EVPhysicalModel:
    """Specifies parameters required for calculation of energy consumption for electric vehicles using physical model.

    Attributes:
        drive_efficiency (float): The proportion of the energy drawn from the battery that is used to move the vehicle.
            (This is to factor in energy losses through heat in the motors, for example.)
            **Note:** This attribute is required when using the physical model `consumptionModel=physical` for EV
            consumption.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        recuperation_efficiency (float): The proportion of the energy gained when braking or going downhill that can be
            recuperated and restored as battery charge.
            **Note:** This attribute is required when using the physical model `consumptionModel=physical` for EV
            consumption.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        auxiliary_power_consumption (float | Unset): Power (in W) consumed by the vehicle's auxiliary systems (for
            example, air conditioning, lights).

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        air_density (float | Unset): The density of air (in kg/m³) used for calculating aerodynamic drag.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
             Default: 1.225.
        auxiliary_power_curve (str | Unset): Function curve specifying power consumed by the vehicle's auxiliary systems
            as a function of time.
            Use this instead of the single-valued `auxiliaryPowerConsumption` when auxiliary power consumption is not
            constant.

            The format of the string is a pipe separated list of number pairs, as follows:

            ```
            <TIME_0>,<POWER_0>|<TIME_1>,<POWER_1>|...|<TIME_N>,<POWER_N>
            ```

            where time values are strictly increasing non-negative integer values in seconds, representing elapsed driving
            time since the start of the route, and power values are non-negative floating point values in watts.
            The function is piecewise constant, e.g., for any elapsed time in the range `[TIME_0, TIME_1]` the value of the
            function is `POWER_0`.
            For time greater than `TIME_N`, the value of the function is `POWER_N`. TIME_0 must be explicitly set to zero.
            The time in the last pair, `TIME_N`, can be at most 1800 (30 minutes).

            Waypoint waiting times, setup times or anything other than driving time does not
            cause power consumption but function curve keeps advancing.

            The list can have a maximum of six `TIME`,`POWER` pairs.

            **Note**: When using this feature `departureTime` needs to be set and cannot be set to `any`.

            **Alpha**: This parameter is in development. It may not be stable and is subject to change.
        initial_charge (float | Unset): Charge level of the vehicle's battery at the start of the route (in kWh).
            Value must be less than or equal to the value of `maxCharge`.
        max_charge (float | Unset): Total capacity of the vehicle's battery (in kWh).
        charging_curve (str | Unset): Function curve describing the maximum battery charging rate (in kW) at a given
            charge level (in kWh).

            The format of the string is a comma-separated list of numbers, as follows:

            ```
            <CHARGE_0>,<RATE_0>,<CHARGE_1>,<RATE_1>,...,<RATE_N>,<CHARGE_N>
            ```

            where charge values are strictly increasing, non-negative floating-point values in units
            of (kWh), and rate values are positive floating point values in units of (kW).

            Charge values must cover the entire range of `[0, maxChargeAfterChargingStation`]. The
            charging curve is piecewise constant, e.g., for any charge in the range `[CHARGE_0,
            CHARGE_1)`, the value of the function is `RATE_0`.

            Maximum value of rate must not be greater than 2000 kW. The values of charge in the curve are only validated up
            to the value of `maxChargeAfterChargingStation` since values outside this range are not used.
            Minimum value of rate must not be less than 7kW. The values of charge in the curve are only validated up to the
            value of `maxChargeAfterChargingStation` since values outside this range are not used.

            The algorithm calculates a route as the best possible combination of driving and charging
            parts and uses the charging curve to evaluate the most efficent range of
            charging. For example, if the rate of charging is high at lower levels of battery, but slows down significantly
            after charging a little, stopping
            often and charging less, but quicker, at each station might be better for the overall route.
            Because batteries lose charging speed with use, providing a charging curve for the
            exact battery would give more accurate charging time estimate than providing a
            generic curve for all batteries of one type.
             Example: 0,207,28.761,207,29.011,173,43.493,173,43.743,145,51.209,145,51.459,113,56.120,113,56.370,87,80.0,10.
        max_charging_voltage (float | Unset): Maximum charging voltage supported by the vehicle's battery (in Volt).
        max_charging_current (float | Unset): Maximum charging current supported by the vehicle's battery (in Ampere).
        max_charge_after_charging_station (float | Unset): Maximum charge to which the battery should be charged at a
            charging station (in kWh).
            Value must be less than or equal to the value of `maxCharge`.

            The algorithm calculates a route as the best possible combination of driving and charging
            parts so charging at a charging station does not happen strictly to the value of
            this parameter. Instead, the algorithm attempts to leave every station with
            different charge levels, and only the best possible combination of charging stations
            and target charge will form the final route.

            For example, if there is a fast but not reachable charging station on the route,
            the algorithm prefers first to charge at a slower station, but only to a level that enables it
            to reach the fast station. This way it calculates the best possible combination of driving
            and charging parts.
        min_charge_at_charging_station (float | Unset): Minimum charge when arriving at a charging station (in kWh).
            Value must be less than the value of `maxChargeAfterChargingStation`.

            The algorithm calculates a route as the best possible combination of driving and charging
            parts so visiting a charging station is planned not when the remaining charge is close
            to the value of this parameter but when it is part of the best possible charging
            plan for the given route.

            For example, it might prefer charging a still half-full battery at the fast charging station because
            there are only slower stations later on the route and the remaining charge is not
            enough to reach the destination without charging at all.
        min_charge_at_first_charging_station (float | Unset): Minimum charge when arriving at first charging station (in
            kWh).
            Value must be less than the value of `maxChargeAfterChargingStation`.

            This overrides `minChargeAtChargingStation` for the first charging station. If not specified,
            `minChargeAtChargingStation`
            will be used for all charging stations, including the first one.

            This is usually used when the current charge is too low to reach a charging station within
            `minChargeAtChargingStation` limits.
        min_charge_at_destination (float | Unset): Minimum charge at the final route destination (in kWh).
            Value must be less than the value of `maxChargeAfterChargingStation`.

            The algorithm calculates a route as the best possible combination of driving and charging
            parts while making sure that the actual value of the charge at the destination would be close to the
            value of this parameter. I.e., the resulting value is expected to be bigger
            than this parameter's value by no more than 10% of the battery capacity.
        charging_setup_duration (int | Unset): Time spent (in seconds) after arriving at a charging station but before
            actually charging
            (for example, time spent for payment processing).
        max_charging_duration (int | Unset): Maximum time spent (in seconds) at a charging station unless otherwise
            specified in a user-introduced charging waypoint. Charging waypoint may
            specify max or min duration, both of which can be higher. (see documentation for the `via` parameter)
        connector_types (str | Unset): Comma-separated list of connector types that are compatible with the vehicle. If
            `makeReachable` is set to `true`,
            then only stations with any of these connector types will be evaluated as a potential charging stop.
            For stations with multiple compatible connectors, the charging time is based on the connector type with the
            highest power rating among them.

            Currently supported connector types are:
              * `iec62196Type1Combo`:  Type 1 Combo connector, commonly called "CCS1"
              * `iec62196Type2Combo`:  Type 2 Combo connector, commonly called "CCS2"
              * `chademo`: CHAdeMO connector
              * `tesla`: Deprecated alias for `saeJ3400`. Use `saeJ3400` instead. Cannot be used in combination with
            `saeJ3400`
              * `saeJ3400`: North American Charging Standard (NACS) connector.
              * `gbtDc`: Guobiao GB/T 20234.3 DC connector
             Example: iec62196Type1Combo,chademo.
        max_power_at_low_voltage (float | Unset): The maximum power (in kW) at which a vehicle can charge given these
            conditions:
            * The charging station connector's maximum supply voltage < 800 V.
            * The vehicle's `maxChargingVoltage` >= 800 V.

            The `consumablePower` at a station is then taken to be the lower of `maxPowerAtLowVoltage` and the charging
            station's available power.
             Default: 45.0.
    """

    drive_efficiency: float
    recuperation_efficiency: float
    auxiliary_power_consumption: float | Unset = UNSET
    air_density: float | Unset = 1.225
    auxiliary_power_curve: str | Unset = UNSET
    initial_charge: float | Unset = UNSET
    max_charge: float | Unset = UNSET
    charging_curve: str | Unset = UNSET
    max_charging_voltage: float | Unset = UNSET
    max_charging_current: float | Unset = UNSET
    max_charge_after_charging_station: float | Unset = UNSET
    min_charge_at_charging_station: float | Unset = UNSET
    min_charge_at_first_charging_station: float | Unset = UNSET
    min_charge_at_destination: float | Unset = UNSET
    charging_setup_duration: int | Unset = UNSET
    max_charging_duration: int | Unset = UNSET
    connector_types: str | Unset = UNSET
    max_power_at_low_voltage: float | Unset = 45.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        drive_efficiency = self.drive_efficiency

        recuperation_efficiency = self.recuperation_efficiency

        auxiliary_power_consumption = self.auxiliary_power_consumption

        air_density = self.air_density

        auxiliary_power_curve = self.auxiliary_power_curve

        initial_charge = self.initial_charge

        max_charge = self.max_charge

        charging_curve = self.charging_curve

        max_charging_voltage = self.max_charging_voltage

        max_charging_current = self.max_charging_current

        max_charge_after_charging_station = self.max_charge_after_charging_station

        min_charge_at_charging_station = self.min_charge_at_charging_station

        min_charge_at_first_charging_station = self.min_charge_at_first_charging_station

        min_charge_at_destination = self.min_charge_at_destination

        charging_setup_duration = self.charging_setup_duration

        max_charging_duration = self.max_charging_duration

        connector_types = self.connector_types

        max_power_at_low_voltage = self.max_power_at_low_voltage

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "driveEfficiency": drive_efficiency,
            "recuperationEfficiency": recuperation_efficiency,
        })
        if auxiliary_power_consumption is not UNSET:
            field_dict["auxiliaryPowerConsumption"] = auxiliary_power_consumption
        if air_density is not UNSET:
            field_dict["airDensity"] = air_density
        if auxiliary_power_curve is not UNSET:
            field_dict["auxiliaryPowerCurve"] = auxiliary_power_curve
        if initial_charge is not UNSET:
            field_dict["initialCharge"] = initial_charge
        if max_charge is not UNSET:
            field_dict["maxCharge"] = max_charge
        if charging_curve is not UNSET:
            field_dict["chargingCurve"] = charging_curve
        if max_charging_voltage is not UNSET:
            field_dict["maxChargingVoltage"] = max_charging_voltage
        if max_charging_current is not UNSET:
            field_dict["maxChargingCurrent"] = max_charging_current
        if max_charge_after_charging_station is not UNSET:
            field_dict["maxChargeAfterChargingStation"] = (
                max_charge_after_charging_station
            )
        if min_charge_at_charging_station is not UNSET:
            field_dict["minChargeAtChargingStation"] = min_charge_at_charging_station
        if min_charge_at_first_charging_station is not UNSET:
            field_dict["minChargeAtFirstChargingStation"] = (
                min_charge_at_first_charging_station
            )
        if min_charge_at_destination is not UNSET:
            field_dict["minChargeAtDestination"] = min_charge_at_destination
        if charging_setup_duration is not UNSET:
            field_dict["chargingSetupDuration"] = charging_setup_duration
        if max_charging_duration is not UNSET:
            field_dict["maxChargingDuration"] = max_charging_duration
        if connector_types is not UNSET:
            field_dict["connectorTypes"] = connector_types
        if max_power_at_low_voltage is not UNSET:
            field_dict["maxPowerAtLowVoltage"] = max_power_at_low_voltage

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        drive_efficiency = d.pop("driveEfficiency")

        recuperation_efficiency = d.pop("recuperationEfficiency")

        auxiliary_power_consumption = d.pop("auxiliaryPowerConsumption", UNSET)

        air_density = d.pop("airDensity", UNSET)

        auxiliary_power_curve = d.pop("auxiliaryPowerCurve", UNSET)

        initial_charge = d.pop("initialCharge", UNSET)

        max_charge = d.pop("maxCharge", UNSET)

        charging_curve = d.pop("chargingCurve", UNSET)

        max_charging_voltage = d.pop("maxChargingVoltage", UNSET)

        max_charging_current = d.pop("maxChargingCurrent", UNSET)

        max_charge_after_charging_station = d.pop(
            "maxChargeAfterChargingStation", UNSET
        )

        min_charge_at_charging_station = d.pop("minChargeAtChargingStation", UNSET)

        min_charge_at_first_charging_station = d.pop(
            "minChargeAtFirstChargingStation", UNSET
        )

        min_charge_at_destination = d.pop("minChargeAtDestination", UNSET)

        charging_setup_duration = d.pop("chargingSetupDuration", UNSET)

        max_charging_duration = d.pop("maxChargingDuration", UNSET)

        connector_types = d.pop("connectorTypes", UNSET)

        max_power_at_low_voltage = d.pop("maxPowerAtLowVoltage", UNSET)

        ev_physical_model = cls(
            drive_efficiency=drive_efficiency,
            recuperation_efficiency=recuperation_efficiency,
            auxiliary_power_consumption=auxiliary_power_consumption,
            air_density=air_density,
            auxiliary_power_curve=auxiliary_power_curve,
            initial_charge=initial_charge,
            max_charge=max_charge,
            charging_curve=charging_curve,
            max_charging_voltage=max_charging_voltage,
            max_charging_current=max_charging_current,
            max_charge_after_charging_station=max_charge_after_charging_station,
            min_charge_at_charging_station=min_charge_at_charging_station,
            min_charge_at_first_charging_station=min_charge_at_first_charging_station,
            min_charge_at_destination=min_charge_at_destination,
            charging_setup_duration=charging_setup_duration,
            max_charging_duration=max_charging_duration,
            connector_types=connector_types,
            max_power_at_low_voltage=max_power_at_low_voltage,
        )

        ev_physical_model.additional_properties = d
        return ev_physical_model

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AutocompleteResultItemLocalityType(str, Enum):
    CITY = "city"
    DISTRICT = "district"
    POSTALCODE = "postalCode"
    SUBDISTRICT = "subdistrict"

    def __str__(self) -> str:
        return str(self.value)



import datetime
import functools
import uuid
from typing import Self

from . import _util


class CasaGeoResult:
    def __init__(self, *, _data: dict, _error: Exception | None = None) -> None:
        self._timestamp = datetime.datetime.now()
        self._uuid = uuid.uuid4()
        self._data = _data
        self._error = _error

    def __bool__(self) -> bool:
        return self._error is None

    def __repr__(self) -> str:
        classname = type(self).__qualname__
        return f"<{classname} {self._uuid} [{'OK' if self else repr(self._error)}]>"

    @classmethod
    def from_response(cls, response: _util.Response) -> Self:
        # NOTE: If load_json() raises an exception, it will be caught by the
        #       decorator surrounding the calling function (if any).
        return cls(_data=_util.load_json(response))

    @classmethod
    def from_exception(cls, exception: Exception) -> Self:
        return cls(_data={}, _error=exception)

    @classmethod
    def wrap_errors(cls):
        """
        Decorator that wraps a function in a try-except block and returns any caught
        exception wrapped in a ``cls`` object.
        """

        def decorator(func):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                try:
                    return func(*args, **kwargs)
                except Exception as err:
                    _util.logger.warning("%s: %s", err.__class__.__name__, err)
                    return cls.from_exception(err)

            return wrapper

        return decorator

    def json(self) -> dict:
        """
        Return the raw API response as a dictionary.

        Returns:
            dict: The raw JSON API response.
        """
        return self._data

    def error(self) -> Exception | None:
        """
        Return the exception that occurred during the API request, if any.

        Returns:
            Exception | None: The exception that occurred, or None if no exception occurred.
        """
        return self._error



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.ev_payment_support_id import EvPaymentSupportId

T = TypeVar("T", bound="EvPaymentSupport")


@_attrs_define
class EvPaymentSupport:
    """
    Attributes:
        id (EvPaymentSupportId): payment method identifier.

            Description of supported values:

            - `authByCarPlugAndCharge`: indicates whether this EV station supports Plug&Charge.ISO 15118 Plug&Charge enables
            drivers to plug in and charge up instantly using automatic EV-to-charging station authentication technology.
            - `creditCard`: indicates if this EV station accepts credit card payment or not.
            - `debitCard`: indicates if this EV station accepts debit card payment or not.
            - `onlineApplePay`: indicates whether this EV station accepts Apple Pay (TM) authentication and payment method.
            - `onlineGooglePay`:  indicates whether this EV station accepts Google Pay (TM) authentication and payment
            method.
            - `onlinePaypal`: indicates whether this EV station accepts PayPal (TM) authentication and payment method.
            - `operatorApp`: indicates whether this EV station support authentication and payment through an app from the
            Charging Point Operator.
        accepted (bool): set to `true` if the related payment method is accepted at the EV station
    """

    id: EvPaymentSupportId
    accepted: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id.value

        accepted = self.accepted

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "accepted": accepted,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = EvPaymentSupportId(d.pop("id"))

        accepted = d.pop("accepted")

        ev_payment_support = cls(
            id=id,
            accepted=accepted,
        )

        ev_payment_support.additional_properties = d
        return ev_payment_support

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
